/* w2ui 1.3.2 (c) http://w2ui.com, vitmalina@gmail.com */
/*************************************************
* --- Reset (used for all w2ui wdigetes)
* --- The reset is needed to coexist with other CSS
* --- on the same page (for example bootstrap)
*/
.w2ui-reset {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-reset * {
  color: default;
  font-family: Verdana, Arial;
  font-size: 11px;
  /* font-weight: normal; - if enabled, then tr cannot have font-weight: bold; */
  line-height: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  margin: 0px;
  padding: 0px;
}
.w2ui-reset table {
  max-width: none;
  background-color: transparent;
  border-collapse: separate;
  border-spacing: 0;
}
.w2ui-reset input,
.w2ui-reset textarea {
  width: auto;
  height: auto;
  vertical-align: baseline;
  padding: 4px;
}
.w2ui-reset select {
  padding: 1px;
  height: 23px;
}
.w2ui-centered {
  display: table;
  width: 100%;
  height: 100%;
}
.w2ui-centered > div {
  display: table-cell;
  padding: 10px 20px 20px 10px;
  text-align: center;
  vertical-align: middle;
}
/*************************************************
* ---- Input Controls ----
*/
input[type=text],
input[type=password],
select,
textarea {
  padding: 4px;
  border: 1px solid #bbbbbb;
  border-radius: 3px;
  color: #000000;
  background-color: #ffffff;
}
input[type=text]:focus,
input[type=password]:focus,
select:focus,
textarea:focus {
  outline-color: #72b2ff;
}
input[type=text]:disabled,
input[type=password]:disabled,
select:disabled,
textarea:disabled,
input[type=text][readonly],
input[type=password][readonly],
select[readonly],
textarea[readonly] {
  background-color: #f1f1f1;
  color: #888;
}
select {
  padding: 2px;
}
/*************************************************
* ---- Overlay and Bubble ----
*/
.w2ui-overlay {
  position: absolute;
  margin-top: 6px;
  margin-left: -17px;
  display: none;
  z-index: 1300;
  color: inherit;
  background-color: #fbfbfb;
  border: 3px solid #777777;
  box-shadow: 0px 2px 10px #999999;
  border-radius: 4px;
  text-align: left;
}
.w2ui-overlay table td {
  color: inherit;
}
.w2ui-overlay:before {
  content: "";
  position: absolute;
  -webkit-transform: rotate(-45deg);
  -moz-transform: rotate(-45deg);
  -ms-transform: rotate(-45deg);
  -o-transform: rotate(-45deg);
  transform: rotate(-45deg);
  width: 10px;
  height: 10px;
  border: 3px solid #777777;
  border-color: inherit;
  background-color: inherit;
  border-left: 1px solid transparent;
  border-bottom: 1px solid transparent;
  border-bottom-left-radius: 50px;
  margin: -9px 0 0 32px;
}
.w2ui-overlay.w2ui-overlay-popup {
  z-index: 1700;
}
.w2ui-tag {
  position: absolute;
  z-index: 1300;
  opacity: 0;
  -webkit-transition: opacity 0.3s;
  -moz-transition: opacity 0.3s;
  -ms-transition: opacity 0.3s;
  -o-transition: opacity 0.3s;
  transition: opacity 0.3s;
}
.w2ui-tag .w2ui-tag-body {
  background-color: rgba(60, 60, 60, 0.82);
  display: inline-block;
  position: absolute;
  border-radius: 4px;
  padding: 4px 10px;
  margin-left: 10px;
  margin-top: 0px;
  color: #ffffff !important;
  box-shadow: 1px 1px 3px #000000;
  line-height: normal;
  font-size: 11px;
  font-family: Verdana, Arial;
}
.w2ui-tag .w2ui-tag-body:before {
  content: "";
  position: absolute;
  width: 0;
  height: 0;
  border-top: 5px solid transparent;
  border-right: 5px solid rgba(60, 60, 60, 0.82);
  border-bottom: 5px solid transparent;
  margin: 2px 0 0 -15px;
}
.w2ui-tag.w2ui-tag-popup {
  z-index: 1700;
}
/*************************************************
* ---- Common Classes ----
*/
.w2ui-marker {
  color: #444;
  background-color: #FFF8B0;
}
.w2ui-spinner {
  display: inline-block;
  background-size: 100%;
  background-repeat: no-repeat;
  background-image: url(data:image/gif;base64, );
}
.w2ui-icon {
  background-repeat: no-repeat;
  height: 16px;
  width: 16px;
  overflow: hidden;
  margin: 2px 2px;
  display: inline-block;
}
.w2ui-icon.icon-reload {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAI/SURBVDjLjZPbS9NhHMYH+zNidtCSQrqwQtY5y2QtT2QGrTZf13TkoYFlzsWa/tzcoR3cSc2xYUlGJfzAaIRltY0N12H5I+jaOxG8De+evhtdOP1hu3hv3sPzPO/z4SsBIPnfuvG8cbBlWiEVO5OUItA0VS8oxi9EdhXo+6yV3V3UGHRvVXHNfNv6zRfNuBZVoiFcB/3LdnQ8U+Gk+bhPVKB3qUOuf6/muaQR/qwDkZ9BRFdCmMr5EPz6BN7lMYylLGgNNaKqt3K0SKDnQ7us690t3rNsxeyvaUz+8OJpzo/QNzd8WTtcaQ7WlBmPvxhx1V2Pg7oDziIBimwwf3qAGWESkVwQ7owNujk1ztvk+cg4NnAUTT4FrrjqUKHdF9jxBfXr1rgjaSk4OlMcLrnOrJ7latxbL1V2lgvlbG9MtMTrMw1r1PImtfyn1n5q47TlBLf90n5NmalMtUdKZoyQMkLKlIGLjMyYhFpmlz3nGEVmFJlRZNaf7pIaEndM24XIjCOzjX9mm2S2JsqdkMYIqbB1j5C6yWzVk7YRFTsGFu7l+4nveExIA9aMCcOJh6DIoMigyOh+o4UryRWQOtIjaJtoziM1FD0mpE4uZcTc72gBaUyYKEI6khgqINXO3saR7kM8IZUVCRDS0Ucf+xFbCReQhr97MZ51wpWxYnhpCD3zOrT4lTisr+AJqVx0Fiiyr4/vhP4VyyMFIUWNqRrV96vWKXKckBoIqWzXYcoPDrUslDJoopuEVEpIB0sR+AuErIiZ6OqMKAAAAABJRU5ErkJggg==) no-repeat center;
}
.w2ui-icon.icon-columns {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAFwSURBVBgZpcG9apNhHMbh3/ORWiztpLSrQ86gmw6egkuXjE7i5CQ0OAZxUSj0DCxiB8/A07BDBXEoKEoGbaP45n/f9im+SwhCyXUl26wiPXjy9vG9+7svpjNv2ICEJUKBZWThCGTjCEJia627ODk5e/ru5d5h3b27ezBY38jb61zHxsUsDoDDOuucZ9NzrutX50yz//rUy4zffPT/PHz+3peotmjGR6f0JqMhven5b2JuYm7C4k+YOzubhIImY9PbvrXJoloqdZCpNzJra5Wbg0qjedDUUND7+v0ni0qGnAo1QSQTWTQK0VTLNJPRkGVKAiVwAnKiqNBIQVOloBkfndKbjIb0Us1UCRkyoMQVSTR53nX0dm5vsSgDKWdKyaScyIUr825OUx2i9+XbDxbllGgElJSwRGOJpkqimYyGLFNSoin8UwqNbJp69unz8aNX3Z4iUIiIQBLRdYyefUA2VuAwsrCERHPMpWSbVWRW9Bcyl/ydiZeC0gAAAABJRU5ErkJggg==) no-repeat center !important;
}
.w2ui-icon.icon-search,
.w2ui-icon.icon-search-down {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACYAAAAgCAYAAAB+ZAqzAAACuElEQVRYw9WXSWhTQRjHR0UKLqhFaV0OUih68GAOWjyJKypCpAoV8aIiioIICiKiB1GMtE3MYmry2moXDz1UDx7sUXHBhQpSaRVxrYpWcMO9avx/8AJh/CbznHkxdeB3Cd/8589kvuUJkWcdjCTHghUgAi6DJ+AVeAqugSQIggniXywcNBJsB70g44EHYBcYXUhTM8EFj4ZkboKqQpiqAv2GprK8o7/f75t6pjn0M3gNPmri3vtycxAZA64qDvkJ2kENqAQTQQWoBg74qth3B4y3NbZDIX4fzNfsnQtuK/YfsjFVCh4pMq3Co0Y5uMVoUGkpy8aFT5xaeSzVEo45bXdBt4LeaLq1k0RXMYJfdDfFmAuAD4zWlty4UNyZEkm19MUb2zMw8Sfp1u+IWSrcIimLnTG8/SijdU6OO5poDESdtgHZVBzUHm/amhW7zoitMTS2mNHqASPk2FDCCcLMYK6p+obmulyxfiYLA4bGKFvfSnrUvkq5+Lpk8z4yRH8r3l/X4WiqJFfspSQ0CGYZGpsMnkt6L+h31Z76hpMdeOwPQ7H0NFnssST0C8wxNDaDKb6kP06150gsHahNNlVzYheZd7HJ0BiX4VRGhpmIhRixKyZilM2M1mnTArtIUbU3/qVO0H0GvmQ4CY4C3YopYYlHjXlggNG4R33Ypi2tVtwaPeTdNMkq9pVQZQdvFPs32zbx4aAjzxhDRfIAWAeWg7VgrzsY5ht/zoNJtubKwA3LITGjSKRyW3NTwaUCmKOSMd3WHH0ZJRQZZkOP1zFKZ3CB++4+aQ6kEeksWAb2a2L7qDv49S1Q6T72MOgEXa6RGFhP3wpS/B6NOWpRs0UxFg7eqTFHjX1hscxtAz/ymEuIYi0cvgF8Y0w5Ro3dZ3M1boJkTaXEUFlug6fsdsRQWzTj0cey+N/Xb2sj5lTh2M6OAAAAAElFTkSuQmCC) no-repeat center !important;
  background-size: 14px 12px !important;
  opacity: 0.9;
}
.w2ui-icon.icon-add {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJvSURBVDjLpZPrS5NhGIf9W7YvBYOkhlkoqCklWChv2WyKik7blnNris72bi6dus0DLZ0TDxW1odtopDs4D8MDZuLU0kXq61CijSIIasOvv94VTUfLiB74fXngup7nvrnvJABJ/5PfLnTTdcwOj4RsdYmo5glBWP6iOtzwvIKSWstI0Wgx80SBblpKtE9KQs/We7EaWoT/8wbWP61gMmCH0lMDvokT4j25TiQU/ITFkek9Ow6+7WH2gwsmahCPdwyw75uw9HEO2gUZSkfyI9zBPCJOoJ2SMmg46N61YO/rNoa39Xi41oFuXysMfh36/Fp0b7bAfWAH6RGi0HglWNCbzYgJaFjRv6zGuy+b9It96N3SQvNKiV9HvSaDfFEIxXItnPs23BzJQd6DDEVM0OKsoVwBG/1VMzpXVWhbkUM2K4oJBDYuGmbKIJ0qxsAbHfRLzbjcnUbFBIpx/qH3vQv9b3U03IQ/HfFkERTzfFj8w8jSpR7GBE123uFEYAzaDRIqX/2JAtJbDat/COkd7CNBva2cMvq0MGxp0PRSCPF8BXjWG3FgNHc9XPT71Ojy3sMFdfJRCeKxEsVtKwFHwALZfCUk3tIfNR8XiJwc1LmL4dg141JPKtj3WUdNFJqLGFVPC4OkR4BxajTWsChY64wmCnMxsWPCHcutKBxMVp5mxA1S+aMComToaqTRUQknLTH62kHOVEE+VQnjahscNCy0cMBWsSI0TCQcZc5ALkEYckL5A5noWSBhfm2AecMAjbcRWV0pUTh0HE64TNf0mczcnnQyu/MilaFJCae1nw2fbz1DnVOxyGTlKeZft/Ff8x1BRssfACjTwQAAAABJRU5ErkJggg==) no-repeat center !important;
}
.w2ui-icon.icon-delete {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAJdSURBVDjLpZP7S1NhGMf9W7YfogSJboSEUVCY8zJ31trcps6zTI9bLGJpjp1hmkGNxVz4Q6ildtXKXzJNbJRaRmrXoeWx8tJOTWptnrNryre5YCYuI3rh+8vL+/m8PA/PkwIg5X+y5mJWrxfOUBXm91QZM6UluUmthntHqplxUml2lciF6wrmdHriI0Wx3xw2hAediLwZRWRkCPzdDswaSvGqkGCfq8VEUsEyPF1O8Qu3O7A09RbRvjuIttsRbT6HHzebsDjcB4/JgFFlNv9MnkmsEszodIIY7Oaut2OJcSF68Qx8dgv8tmqEL1gQaaARtp5A+N4NzB0lMXxon/uxbI8gIYjB9HytGYuusfiPIQcN71kjgnW6VeFOkgh3XcHLvAwMSDPohOADdYQJdF1FtLMZPmslvhZJk2ahkgRvq4HHUoWHRDqTEDDl2mDkfheiDgt8pw340/EocuClCuFvboQzb0cwIZgki4KhzlaE6w0InipbVzBfqoK/qRH94i0rgokSFeO11iBkp8EdV8cfJo0yD75aE2ZNRvSJ0lZKcBXLaUYmQrCzDT6tDN5SyRqYlWeDLZAg0H4JQ+Jt6M3atNLE10VSwQsN4Z6r0CBwqzXesHmV+BeoyAUri8EyMfi2FowXS5dhd7doo2DVII0V5BAjigP89GEVAtda8b2ehodU4rNaAW+dGfzlFkyo89GTlcrHYCLpKD+V7yeeHNzLjkp24Uu1Ed6G8/F8qjqGRzlbl2H2dzjpMg1KdwsHxOlmJ7GTeZC/nesXbeZ6c9OYnuxUc3fmBuFft/Ff8xMd0s65SXIb/gAAAABJRU5ErkJggg==) no-repeat center !important;
}
.w2ui-icon.icon-save {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKfSURBVDjLpZPrS1NhHMf9O3bOdmwDCWREIYKEUHsVJBI7mg3FvCxL09290jZj2EyLMnJexkgpLbPUanNOberU5taUMnHZUULMvelCtWF0sW/n7MVMEiN64AsPD8/n83uucQDi/id/DBT4Dolypw/qsz0pTMbj/WHpiDgsdSUyUmeiPt2+V7SrIM+bSss8ySGdR4abQQv6lrui6VxsRonrGCS9VEjSQ9E7CtiqdOZ4UuTqnBHO1X7YXl6Daa4yGq7vWO1D40wVDtj4kWQbn94myPGkCDPdSesczE2sCZShwl8CzcwZ6NiUs6n2nYX99T1cnKqA2EKui6+TwphA5k4yqMayopU5mANV3lNQTBdCMVUA9VQh3GuDMHiVcLCS3J4jSLhCGmKCjBEx0xlshjXYhApfMZRP5CyYD+UkG08+xt+4wLVQZA1tzxthm2tEfD3JxARH7QkbD1ZuozaggdZbxK5kAIsf5qGaKMTY2lAU/rH5HW3PLsEwUYy+YCcERmIjJpDcpzb6l7th9KtQ69fi09ePUej9l7cx2DJbD7UrG3r3afQHOyCo+V3QQzE35pvQvnAZukk5zL5qRL59jsKbPzdheXoBZc4saFhBS6AO7V4zqCpiawuptwQG+UAa7Ct3UT0hh9p9EnXT5Vh6t4C22QaUDh6HwnECOmcO7K+6kW49DKqS2DrEZCtfuI+9GrNHg4fMHVSO5kE7nAPVkAxKBxcOzsajpS4Yh4ohUPPWKTUh3PaQEptIOr6BiJjcZXCwktaAGfrRIpwblqOV3YKdhfXOIvBLeREWpnd8ynsaSJoyESFphwTtfjN6X1jRO2+FxWtCWksqBApeiFIR9K6fiTpPiigDoadqCEag5YUFKl6Yrciw0VOlhOivv/Ff8wtn0KzlebrUYwAAAABJRU5ErkJggg==) no-repeat center !important;
}
.w2ui-icon.icon-edit {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAFUSURBVDjLrZM/SAJxGIZdWwuDlnCplkAEm1zkaIiGFFpyMIwGK5KGoK2lphDKkMDg3LLUSIJsSKhIi+684CokOtTiMizCGuzEU5K3vOEgKvtBDe/2Pc8H3x8NAM1fQlx4H9M3pcOWp6TXWmM8A7j0629v1nraiAVC0IrrwATKIgs5xyG5QiE+Z4iQdoeU2oAsnqCSO1NSTu+D9VhqRLD8nIB8F0Q2MgmJDyipCzjvYJkIfpN2UBLG8MpP4dxvQ3ZzGuyyBQ2H+AnOOCBd9aL6soh81A5hyYSGWyCFvxUcerqI4S+CvYVOFPMHxLAq8I3qdHVY5LbBhJzEsCrwutpRFBlUHy6wO2tEYtWAzLELPN2P03kjfj3luqDycV2F8AgefWbEnVqEHa2IznSD6BdsVDNStB0lfh0FPoQjdx8RrAqGzC0YprSgxzsUMOY2bf37N/6Ud1Vc9yYcH50CAAAAAElFTkSuQmCC) no-repeat center !important;
}
.w2ui-icon.icon-bullet-black {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAQAAAC1+jfqAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAABlSURBVCjPY/jPgB8yDC4FilKKDfJXZa6KNwhKYVfQkPW/63/b/+T/XA1YFchd7fqf/j/2f+N/5qtYFUhe7fif9D/sf91/BuwKhBoS/jcBpcP/M2C3gluKrYHhKhA2MEgN2pDEDgEb0f5zlvXgVgAAAABJRU5ErkJggg==) no-repeat center !important;
}
.w2ui-icon.icon-folder {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGrSURBVDjLxZO7ihRBFIa/6u0ZW7GHBUV0UQQTZzd3QdhMQxOfwMRXEANBMNQX0MzAzFAwEzHwARbNFDdwEd31Mj3X7a6uOr9BtzNjYjKBJ6nicP7v3KqcJFaxhBVtZUAK8OHlld2st7Xl3DJPVONP+zEUV4HqL5UDYHr5xvuQAjgl/Qs7TzvOOVAjxjlC+ePSwe6DfbVegLVuT4r14eTr6zvA8xSAoBLzx6pvj4l+DZIezuVkG9fY2H7YRQIMZIBwycmzH1/s3F8AapfIPNF3kQk7+kw9PWBy+IZOdg5Ug3mkAATy/t0usovzGeCUWTjCz0B+Sj0ekfdvkZ3abBv+U4GaCtJ1iEm6ANQJ6fEzrG/engcKw/wXQvEKxSEKQxRGKE7Izt+DSiwBJMUSm71rguMYhQKrBygOIRStf4TiFFRBvbRGKiQLWP29yRSHKBTtfdBmHs0BUpgvtgF4yRFR+NUKi0XZcYjCeCG2smkzLAHkbRBmP0/Uk26O5YnUActBp1GsAI+S5nRJJJal5K1aAMrq0d6Tm9uI6zjyf75dAe6tx/SsWeD//o2/Ab6IH3/h25pOAAAAAElFTkSuQmCC) no-repeat center !important;
}
.w2ui-icon.icon-page {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAINSURBVBgZBcG/r55zGAfg6/4+z3va01NHlYgzEfE7MdCIGISFgS4Gk8ViYyM2Mdlsko4GSf8Do0FLRCIkghhYJA3aVBtEz3nP89wf11VJvPDepdd390+8Nso5nESBQoq0pfvXm9fzWf19453LF85vASqJlz748vInb517dIw6EyYBIIG49u+xi9/c9MdvR//99MPPZ7+4cP4IZhhTPbwzT2d+vGoaVRRp1rRliVvHq+cfvM3TD82+7mun0o/ceO7NT+/4/KOXjwZU1ekk0840bAZzMQ2mooqh0A72d5x/6sB9D5zYnff3PoYBoWBgFKPKqDKqjCpjKr//dcu9p489dra88cydps30KswACfNEKanSaxhlntjJ8Mv12Paie+vZ+0+oeSwwQ0Iw1xAR1CiFNJkGO4wu3ZMY1AAzBI0qSgmCNJsJUEOtJSMaCTBDLyQ0CknAGOgyTyFFiLI2awMzdEcSQgSAAKVUmAeNkxvWJWCGtVlDmgYQ0GFtgg4pNtOwbBcwQy/Rife/2yrRRVI0qYCEBly8Z+P4qMEMy7JaVw72N568e+iwhrXoECQkfH91kY7jwwXMsBx1L93ZruqrK6uuiAIdSnTIKKPLPFcvay8ww/Hh+ufeznTXu49v95IMoQG3784gYXdTqvRmqn/Wpa/ADFX58MW3L71SVU9ETgEIQQQIOOzub+fhIvwPRDgeVjWDahIAAAAASUVORK5CYII=) no-repeat center !important;
}
/*************************************************
* ---- Locking portion of the screen (in grid, form, layout)
*/
.w2ui-lock {
  opacity: 0.15;
  filter: alpha(opacity=15);
  background-color: #333333;
  position: absolute;
  z-index: 1400;
  display: none;
}
.w2ui-lock-msg {
  position: absolute;
  z-index: 1400;
  width: 200px;
  height: 80px;
  padding: 30px 8px;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
  font-size: 13px;
  font-family: Verdana, Arial;
  opacity: 0.8;
  filter: alpha(opacity=80);
  background-color: #555555;
  color: #ffffff;
  text-align: center;
  border-radius: 5px;
  border: 2px solid #444444;
  display: none;
}
.w2ui-lock-msg .w2ui-spinner {
  display: inline-block;
  width: 24px;
  height: 24px;
  margin: -3px 8px -7px -10px;
}
/*************************************************
* ---- Forms ----
*/
.w2ui-form {
  position: relative;
  color: #000000;
  background-color: #f5f6f7;
  border: 1px solid #c0c0c0;
  border-radius: 3px;
  padding: 0px;
  overflow: hidden !important;
}
.w2ui-form > div {
  position: absolute;
  overflow: hidden;
}
.w2ui-form .w2ui-form-header {
  position: absolute;
  left: 0;
  right: 0;
  border-bottom: 1px solid #99bbe8 !important;
  overflow: hidden;
  color: #444444;
  font-size: 13px;
  text-align: center;
  padding: 8px;
  background-image: -webkit-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -moz-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -ms-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -o-linear-gradient(#dae6f3, #c2d5ed);
  background-image: linear-gradient(#dae6f3, #c2d5ed);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdae6f3', endColorstr='#ffc2d5ed', GradientType=0);
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
}
.w2ui-form .w2ui-form-toolbar {
  position: absolute;
  left: 0px;
  right: 0px;
  margin: 0px;
  padding: 6px 3px;
  border-bottom: 1px solid #d5d8d8;
}
.w2ui-form .w2ui-form-tabs {
  margin: 0px;
  padding: 0px;
}
.w2ui-form .w2ui-tabs {
  position: absolute;
  left: 0;
  right: 0;
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  padding-top: 5px !important;
  background-color: #fafafa;
}
.w2ui-form .w2ui-tabs .w2ui-tab.active {
  background-color: #f5f6f7;
}
.w2ui-form .w2ui-page {
  position: absolute;
  left: 0;
  right: 0;
  overflow: auto;
  padding: 10px;
  border-left: 1px solid inherit;
  border-right: 1px solid inherit;
  background-color: inherit;
  border-radius: 3px;
}
.w2ui-form .w2ui-buttons {
  position: absolute;
  left: 0;
  right: 0;
  bottom: 0;
  text-align: center;
  border-top: 1px solid #d5d8d8;
  border-bottom: 0px solid #d5d8d8;
  background-color: #fafafa;
  padding: 15px 0px !important;
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
}
.w2ui-form .w2ui-buttons input[type="button"],
.w2ui-form .w2ui-buttons button {
  width: 80px;
  margin-right: 5px;
}
.w2ui-form input[type=checkbox] {
  margin-top: 4px;
  margin-bottom: 4px;
}
.w2ui-group {
  background-color: #ebecef;
  margin: 5px 0px 10px 0px;
  padding: 10px 5px;
  border-top: 1px solid #cedcea;
  border-bottom: 1px solid #cedcea;
}
.w2ui-label {
  float: left;
  margin-top: 6px;
  margin-bottom: 3px;
  width: 120px;
  padding: 0px;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  text-align: right;
  min-height: 20px;
}
.w2ui-label.w2ui-span1 {
  width: 20px;
  float: left;
}
.w2ui-label.w2ui-span2 {
  width: 40px;
  float: left;
}
.w2ui-label.w2ui-span3 {
  width: 60px;
  float: left;
}
.w2ui-label.w2ui-span4 {
  width: 80px;
  float: left;
}
.w2ui-label.w2ui-span5 {
  width: 100px;
  float: left;
}
.w2ui-label.w2ui-span6 {
  width: 120px;
  float: left;
}
.w2ui-label.w2ui-span7 {
  width: 140px;
  float: left;
}
.w2ui-label.w2ui-span8 {
  width: 160px;
  float: left;
}
.w2ui-label.w2ui-span9 {
  width: 180px;
  float: left;
}
.w2ui-label.w2ui-span10 {
  width: 200px;
  float: left;
}
.w2ui-field {
  /* do not include width */
  margin-bottom: 3px;
  margin-left: 128px;
  padding: 3px;
  min-height: 20px;
}
.w2ui-field.w2ui-span1 {
  margin-left: 28px;
  float: none;
}
.w2ui-field.w2ui-span2 {
  margin-left: 48px;
  float: none;
}
.w2ui-field.w2ui-span3 {
  margin-left: 68px;
  float: none;
}
.w2ui-field.w2ui-span4 {
  margin-left: 88px;
  float: none;
}
.w2ui-field.w2ui-span5 {
  margin-left: 108px;
  float: none;
}
.w2ui-field.w2ui-span6 {
  margin-left: 128px;
  float: none;
}
.w2ui-field.w2ui-span7 {
  margin-left: 148px;
  float: none;
}
.w2ui-field.w2ui-span8 {
  margin-left: 168px;
  float: none;
}
.w2ui-field.w2ui-span9 {
  margin-left: 188px;
  float: none;
}
.w2ui-field.w2ui-span10 {
  margin-left: 208px;
  float: none;
}
.w2ui-required {
  position: relative;
}
.w2ui-required::before {
  content: '*';
  position: absolute;
  margin-top: 5px;
  margin-left: -9px;
  color: #ff0000;
}
.w2ui-error {
  border: 1px solid #ffa8a8 !important;
  background-color: #fff4eb !important;
}
/* create an arrow that points up */
.w2ui-field-helper {
  position: absolute;
  display: inline-block;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
  /* create an arrow that points down */
  /* create an arrow that points left */
  /* create an arrow that points right */
}
.w2ui-field-helper .w2ui-field-up {
  position: absolute;
  top: 0px;
  padding: 2px 3px;
}
.w2ui-field-helper .w2ui-field-down {
  position: absolute;
  bottom: 0px;
  padding: 2px 3px;
}
.w2ui-field-helper .arrow-up:hover {
  border-bottom-color: #EECCCC;
}
.w2ui-field-helper .arrow-down:hover {
  border-top-color: #EECCCC;
}
.w2ui-field-helper .arrow-up {
  width: 0;
  height: 0;
  border-left: 4px solid transparent;
  /* left arrow slant */
  border-right: 4px solid transparent;
  /* right arrow slant */
  border-bottom: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
.w2ui-field-helper .arrow-down {
  width: 0;
  height: 0;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent;
  border-top: 5px solid #777;
  font-size: 0;
  line-height: 0;
}
.w2ui-field-helper .arrow-left {
  width: 0;
  height: 0;
  border-bottom: 4px solid transparent;
  /* left arrow slant */
  border-top: 4px solid transparent;
  /* right arrow slant */
  border-right: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
.w2ui-field-helper .arrow-right {
  width: 0;
  height: 0;
  border-bottom: 4px solid transparent;
  /* left arrow slant */
  border-top: 4px solid transparent;
  /* right arrow slant */
  border-left: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
.w2ui-color {
  padding: 5px;
  background-color: white;
  border-radius: 3px;
}
.w2ui-color > table {
  table-layout: fixed;
  width: 160px;
}
.w2ui-color > table td {
  width: 20px;
  height: 20px;
}
.w2ui-color > table td div {
  cursor: pointer;
  display: inline-block;
  width: 16px;
  height: 17px;
  padding: 1px 4px;
  border: 1px solid transparent;
  color: white;
  text-shadow: 0px 0px 2px #000;
}
.w2ui-color > table td div:hover {
  outline: 1px solid #666;
  border: 1px solid #fff;
}
.w2ui-calendar {
  position: absolute;
  z-index: 1600;
  display: none;
  margin: 3px 0px 0px -2px;
  padding: 0px;
  border: 1px solid #aaaaaa;
  border-radius: 4px;
  box-shadow: 2px 2px 8px #bbbbbb;
  line-height: 108%;
}
.w2ui-calendar .w2ui-calendar-title {
  padding: 5px 1px 6px 1px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  background-image: -webkit-linear-gradient(#e6e6e6, #c2c4c4);
  background-image: -moz-linear-gradient(#e6e6e6, #c2c4c4);
  background-image: -ms-linear-gradient(#e6e6e6, #c2c4c4);
  background-image: -o-linear-gradient(#e6e6e6, #c2c4c4);
  background-image: linear-gradient(#e6e6e6, #c2c4c4);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffe6e6e6', endColorstr='#ffc2c4c4', GradientType=0);
  border-bottom: 1px solid #999999;
  color: #000000;
  text-align: center;
  font-weight: bold;
}
.w2ui-calendar .w2ui-calendar-previous,
.w2ui-calendar .w2ui-calendar-next {
  width: 24px;
  height: 20px;
  color: #666666;
  border: 1px solid transparent;
  border-radius: 3px;
  padding: 2px 3px 1px 2px;
  margin: -4px 0px 0px 0px;
  cursor: default;
}
.w2ui-calendar .w2ui-calendar-previous:hover,
.w2ui-calendar .w2ui-calendar-next:hover {
  border: 1px solid #c0c0c0;
  background-color: #efefef;
}
.w2ui-calendar .w2ui-calendar-previous {
  float: left;
}
.w2ui-calendar .w2ui-calendar-next {
  float: right;
}
.w2ui-calendar table.w2ui-calendar-days {
  border-bottom-left-radius: 4px;
  border-bottom-right-radius: 4px;
  padding: 0px;
}
.w2ui-calendar table.w2ui-calendar-days td {
  border: 1px solid #e8e8e8;
  color: #000000;
  background-color: #f4f4fe;
  padding: 5px;
  cursor: default;
  text-align: right;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-saturday,
.w2ui-calendar table.w2ui-calendar-days td.w2ui-sunday {
  border: 1px solid #e8e8e8;
  color: #c8493b;
  background-color: #f4f4fe;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-saturday:hover,
.w2ui-calendar table.w2ui-calendar-days td.w2ui-sunday:hover {
  border: 1px solid #aaaaaa;
  color: #000000;
  background-color: #ffff00;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-today {
  border: 1px solid #8cb067;
  color: #000000;
  background-color: #e2f7cd;
}
.w2ui-calendar table.w2ui-calendar-days td:hover {
  border: 1px solid #aaaaaa;
  color: #000000;
  background-color: #ffff00;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-blocked-date {
  text-decoration: line-through;
  border: 1px solid #e8e8e8;
  color: #808080;
  background-color: #e8e8e8;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-day-empty {
  border: 1px solid #e8e8e8;
  background-color: #f4f4fe;
}
.w2ui-calendar table.w2ui-calendar-days tr.w2ui-day-title td {
  border: 1px solid #e8e8e8;
  color: #808080;
  background-color: #e8e8e8;
  text-align: center;
  padding: 5px;
}
.w2ui-list {
  color: inherit;
  position: absolute;
  padding: 0px;
  margin: 0px;
  min-height: 25px;
  overflow: auto;
  border: 1px solid #c0c0c0;
  border-radius: 3px;
  font-size: 6px;
  line-height: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  background-color: #ffffff;
}
.w2ui-list input[type=text] {
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  -ms-box-shadow: none;
  -o-box-shadow: none;
  box-shadow: none;
}
.w2ui-list ul {
  list-style-type: none;
  background-color: black;
  margin: 0px;
  padding: 0px;
}
.w2ui-list ul li {
  float: left;
  margin: 2px 1px 0px 2px;
  border-radius: 3px;
  width: auto;
  padding: 1px 5px;
  border: 1px solid #88b0d6;
  background-color: #eff3f5;
  white-space: nowrap;
  cursor: default;
  font-family: verdana;
  font-size: 11px;
  line-height: normal;
}
.w2ui-list ul li:hover {
  background-color: #EECCCC;
}
.w2ui-list ul li:last-child {
  border-radius: 0px;
  border: 1px solid transparent;
  background-color: transparent;
}
.w2ui-list ul li:last-child input {
  padding: 0px;
  margin: 0px;
  outline: none;
  border: 0px;
}
.w2ui-list ul li div {
  float: right;
  width: 15px;
  height: 16px;
  margin: -1px -3px 0px 4px;
  border-radius: 15px;
}
.w2ui-list ul li div:hover {
  background-color: #EECCCC;
  color: white;
}
.w2ui-list ul li div:before {
  position: relative;
  top: 1px;
  left: 4px;
  color: inherit;
  opacity: 0.6;
  text-shadow: inherit;
  content: 'x';
}
.w2ui-list ul li input {
  width: 10px;
  background-color: transparent;
}
.w2ui-items {
  position: absolute;
  margin-top: 6px;
  display: none;
  z-index: 1600;
  background-color: #ffffff;
  border: 3px solid #777777;
  box-shadow: 0px 3px 8px #999999;
  border-radius: 4px;
  text-align: left;
}
.w2ui-items:before {
  content: "";
  position: absolute;
  z-index: 1601;
  width: 0px;
  height: 0px;
  border-left: 7px solid transparent;
  border-right: 7px solid transparent;
  border-bottom: 7px solid transparent;
  border-bottom-color: inherit;
  margin: -8px 0 0 5px;
}
.w2ui-items .w2ui-items-list {
  border-radius: 4px;
  padding: 3px;
}
.w2ui-items .w2ui-empty-list {
  padding: 5px 2px;
  text-align: center;
  color: #808080;
}
.w2ui-items ul li {
  display: block;
  -webkit-transition: all 0.2s;
  -moz-transition: all 0.2s;
  -ms-transition: all 0.2s;
  -o-transition: all 0.2s;
  transition: all 0.2s;
  padding: 5px;
  cursor: pointer;
}
.w2ui-items ul li.w2ui-item-even {
  color: inherit;
  background-color: #ffffff;
}
.w2ui-items ul li.w2ui-item-odd {
  color: inherit;
  background-color: #f3f6fa;
}
.w2ui-items ul li.w2ui-item-group {
  color: #444;
  font-weight: bold;
  background-color: #ECEDF0;
  border-bottom: 1px solid #D3D2D4;
}
.w2ui-items ul li.selected {
  color: inherit;
  background-color: #b6d5ff;
}
.w2ui-upload {
  position: absolute;
  background-color: #ffffff;
  border: 1px solid #c0c0c0 !important;
  border-radius: 3px;
  text-align: center;
  overflow: auto;
}
.w2ui-upload > span:first-child {
  pointer-events: none;
  margin: 0px;
  padding: 0px;
  font-size: 14px !important;
  color: #999;
}
.w2ui-upload.dragover {
  background-color: #D9FFD5;
  border: 1px solid #93E07D;
}
.w2ui-upload .file-list {
  margin: 0px;
  padding: 0px;
  margin-bottom: 6px;
}
.w2ui-upload .file-list li {
  padding: 2px 5px;
  font-family: verdana;
  font-size: 11px;
  line-height: normal;
  float: left;
  display: inline-block;
  border: 1px solid #88b0d6;
  background-color: #eff3f5;
  border-radius: 3px;
  margin: 2px 0px 2px 2px;
  text-align: left;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
  cursor: default;
  width: auto;
  max-width: 400px;
}
.w2ui-upload .file-list li:hover {
  background-color: #EECCCC;
}
.w2ui-upload .file-list li > span.file-size {
  pointer-events: none;
  color: #777;
}
.w2ui-upload .file-list li > span.file-name {
  pointer-events: none;
}
.w2ui-upload .file-list li > div.file-delete {
  float: right;
  z-index: 1;
  width: 16px;
  height: 16px;
  margin: -1px -4px 0px 4px;
  border-radius: 15px;
}
.w2ui-upload .file-list li > div.file-delete:hover {
  background-color: #EECCCC;
  color: white;
}
.w2ui-upload .file-list li > div.file-delete:before {
  position: relative;
  top: 1px;
  left: 5px;
  opacity: 0.6;
  color: inherit;
  text-shadow: inherit;
  content: 'x';
}
/*************************************************
* ---- Layout ----
*/
.w2ui-layout {
  overflow: hidden !important;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout > div {
  position: absolute;
  overflow: hidden;
  border: 0px;
  margin: 0px;
  padding: 0px;
  outline: 0px;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout > div .w2ui-panel {
  display: none;
  position: absolute;
  z-index: 120;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-tabs {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  z-index: 2;
  display: none;
  overflow: hidden;
  background-color: #fafafa;
  padding: 4px 0px;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-tabs > .w2ui-tab.active {
  background-color: #f5f6f7;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-toolbar {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  z-index: 2;
  display: none;
  overflow: hidden;
  background-color: #fafafa;
  border-bottom: 1px solid silver;
  padding: 4px;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-content {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  bottom: 0px;
  z-index: 1;
  color: inherit;
  background-color: #f5f6f7;
}
.w2ui-layout > div .w2ui-resizer {
  display: none;
  position: absolute;
  z-index: 121;
  background-color: transparent;
}
.w2ui-layout > div .w2ui-resizer:hover,
.w2ui-layout > div .w2ui-resizer.active {
  background-color: #EECCCC;
}
/*************************************************
* ---- Grid ----
*/
.w2ui-grid {
  position: relative;
  border: 1px solid #c0c0c0;
  border-radius: 2px;
  overflow: hidden !important;
}
.w2ui-grid > div {
  position: absolute;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-header {
  position: absolute;
  border-bottom: 1px solid #99bbe8 !important;
  height: 28px;
  overflow: hidden;
  color: #444444;
  font-size: 13px;
  text-align: center;
  padding: 7px;
  background-image: -webkit-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -moz-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -ms-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -o-linear-gradient(#dae6f3, #c2d5ed);
  background-image: linear-gradient(#dae6f3, #c2d5ed);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdae6f3', endColorstr='#ffc2d5ed', GradientType=0);
  border-top-left-radius: 2px;
  border-top-right-radius: 2px;
}
.w2ui-grid .w2ui-grid-toolbar {
  position: absolute;
  border-bottom: 1px solid #c0c0c0;
  background-color: #dee0e4;
  height: 38px;
  padding: 7px 3px 4px 3px;
  margin: 0px;
  box-shadow: 0px 1px 2px #dddddd;
}
.w2ui-grid .w2ui-toolbar-search {
  width: 160px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-all {
  outline: none;
  width: 160px;
  border-radius: 10px;
  line-height: 100%;
  height: 22px;
  border: 1px solid #999999;
  color: #000000;
  background-color: #ffffff;
  padding: 3px 18px 3px 23px;
  margin: 0px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-down {
  position: absolute;
  margin-top: -7px;
  margin-left: 6px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear {
  position: absolute;
  width: 16px;
  height: 16px;
  margin-top: -8px;
  margin-left: -20px;
  border-radius: 15px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear:hover {
  background-color: #EECCCC;
  color: white;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear:before {
  position: relative;
  top: 1px;
  left: 5px;
  opacity: 0.6;
  color: inherit;
  text-shadow: inherit;
  content: 'x';
}
.w2ui-grid .w2ui-grid-body {
  position: absolute;
  overflow: hidden;
  padding: 0px;
  background-color: #ffffff;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-grid .w2ui-grid-body input,
.w2ui-grid .w2ui-grid-body select,
.w2ui-grid .w2ui-grid-body textarea {
  -webkit-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  -o-user-select: text;
  user-select: text;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns {
  overflow: hidden;
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  box-shadow: 0px 1px 4px #dddddd;
  height: auto;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns table {
  height: auto;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns .w2ui-resizer {
  position: absolute;
  display: block;
  background-image: none;
  background-color: rgba(0, 0, 0, 0);
  /* needed for IE */
  padding: 0px;
  margin: 0px;
  width: 6px;
  height: 12px;
  cursor: col-resize;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 0px;
  bottom: 0px;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd {
  color: inherit;
  background-color: #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd:hover {
  color: inherit;
  background-color: #EECCCC;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd.w2ui-empty-record:hover {
  background-color: #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even {
  color: inherit;
  background-color: #f3f6fa;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even:hover {
  color: inherit;
  background-color: #EECCCC;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even.w2ui-empty-record:hover {
  background-color: #f3f6fa;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-selected,
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-selected {
  color: #000000 !important;
  background-color: #b6d5ff !important;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded {
  background-color: #CCDCF0 !important;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded1 {
  height: 0px;
  border-bottom: 1px solid #b2bac0;
  background-color: #CCDCF0;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded1 > div {
  height: 100%;
  margin: 0px;
  padding: 0px;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded2 {
  height: 0px;
  border-radius: 0px;
  border-bottom: 1px solid #b2bac0;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded2 > div {
  height: 0px;
  border: 0px;
  transition: height .3s, opacity .3s;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more {
  border-top: 1px solid #d6d5d7;
  cursor: pointer;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more > div {
  text-align: center;
  color: #777777;
  background-color: rgba(233, 237, 243, 0.5);
  padding: 10px 0px 15px 0px;
  border-top: 1px solid #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more > div:hover {
  color: inherit;
  background-color: #EECCCC;
}
.w2ui-grid .w2ui-grid-body table {
  border-spacing: 0px;
  border-collapse: collapse;
  table-layout: fixed;
  width: 1px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head {
  margin: 0px;
  padding: 0px;
  border-right: 1px solid #c5c5c5;
  border-bottom: 1px solid #c5c5c5;
  color: #000000;
  background-image: -webkit-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -moz-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -ms-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -o-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: linear-gradient(#f9f9f9, #e4e4e4);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fff9f9f9', endColorstr='#ffe4e4e4', GradientType=0);
}
.w2ui-grid .w2ui-grid-body table .w2ui-head > div {
  padding: 7px 3px;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body table td {
  border-right: 1px solid #d6d5d7;
  border-bottom: 0px solid #d6d5d7;
  cursor: default;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data {
  margin: 0px;
  padding: 0px;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data > div {
  padding: 3px 3px 3px 3px;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data > div.flexible-record {
  height: auto;
  overflow: visible;
  white-space: normal;
}
.w2ui-grid .w2ui-grid-body table td:last-child {
  border-right: 0px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-number {
  width: 34px;
  color: #777777;
  background-color: rgba(233, 237, 243, 0.5);
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-number div {
  padding: 0px 7px 0px 3px;
  text-align: right;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select {
  width: 26px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select div {
  padding: 0px 0px;
  text-align: center;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select div input[type=checkbox] {
  margin-top: 2px;
  position: relative;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-expand {
  width: 26px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-expand div {
  padding: 0px 0px;
  text-align: center;
  font-weight: bold;
}
.w2ui-grid .w2ui-grid-body div.w2ui-sort-up {
  background: url(data:image/png;base64,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) no-repeat center right !important;
  height: auto !important;
  width: 100%;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body div.w2ui-sort-down {
  background: url(data:image/png;base64,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) no-repeat center right !important;
  height: auto !important;
  width: 100%;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body .w2ui-col-group {
  text-align: center;
}
.w2ui-grid .w2ui-changed {
  background: url(data:image/gif;base64,R0lGODlhCgAKAJEAALAABf///wAAAAAAACH5BAEAAAIALAAAAAAKAAoAAAIPlI8Hy8mbxIsSUnup3rQAADs=) no-repeat top right;
}
.w2ui-grid .w2ui-editable {
  overflow: hidden;
  height: 100% !important;
  margin: 0px !important;
  padding: 0px !important;
}
.w2ui-grid .w2ui-editable input {
  border: 0px;
  border-radius: 0px;
  margin: 0px;
  padding: 4px 3px;
  width: 100%;
  height: 100%;
}
.w2ui-grid .w2ui-grid-summary {
  position: absolute;
  box-shadow: 0px -1px 4px #aaaaaa;
}
.w2ui-grid .w2ui-grid-summary table {
  color: inherit;
}
.w2ui-grid .w2ui-grid-summary table .w2ui-odd {
  background-color: #eef5eb;
}
.w2ui-grid .w2ui-grid-summary table .w2ui-even {
  background-color: #f8fff5;
}
.w2ui-grid .w2ui-grid-footer {
  position: absolute;
  margin: 0px;
  padding: 0px;
  text-align: center;
  height: 24px;
  overflow: hidden;
  -webkit-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  -o-user-select: text;
  user-select: text;
  box-shadow: 0px -1px 4px #eeeeee;
  color: #444444;
  background-color: #f8f8f8;
  border-top: 1px solid #dddddd;
  border-bottom-left-radius: 2px;
  border-bottom-right-radius: 2px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-left {
  float: left;
  padding-top: 5px;
  padding-left: 5px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-right {
  float: right;
  padding-top: 5px;
  padding-right: 5px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center {
  padding: 2px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav {
  width: 110px;
  margin: 0 auto;
  padding: 0px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav input[type=text] {
  padding: 1px 2px 2px 2px;
  border-radius: 3px;
  width: 40px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav a.w2ui-footer-btn {
  display: inline-block;
  border-radius: 3px;
  cursor: pointer;
  font-size: 11px;
  line-height: 16px;
  padding: 1px 5px;
  width: 30px;
  height: 18px;
  margin-top: -1px;
  color: #000000;
  background-color: transparent;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav a.w2ui-footer-btn:hover {
  color: #000000;
  background-color: #EECCCC;
}
/* SpeadSheet */
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd:hover,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even:hover {
  background-color: inherit;
}
.w2ui-ss .w2ui-grid-records table td {
  border-right-width: 1px;
  border-bottom: 1px solid #efefef;
}
.w2ui-ss .w2ui-grid-records table tr:first-child td {
  border-bottom: 0px;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-selected,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-selected {
  background-color: #EEF4FE !important;
}
.w2ui-ss .w2ui-changed {
  background: inherit;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection {
  position: absolute;
  border: 2px solid #6299DA;
  /* #457FC2; */
  pointer-events: none;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection .w2ui-selection-resizer {
  cursor: crosshair;
  position: absolute;
  bottom: 0px;
  right: 0px;
  width: 6px;
  height: 6px;
  margin-right: -3px;
  margin-bottom: -3px;
  background-color: #457FC2;
  border: 0.5px solid #fff;
  outline: 1px solid white;
  pointer-events: auto;
}
.w2ui-overlay .w2ui-select-field {
  padding: 8px 5px;
  cursor: default;
}
.w2ui-overlay .w2ui-select-field table {
  font-size: 11px;
  font-family: Verdana, Arial;
  border-spacing: 0px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-select-field table tr:hover {
  background-color: #EECCCC;
}
.w2ui-overlay .w2ui-select-field table td:nth-child(1) {
  padding: 3px 3px 3px 6px;
}
.w2ui-overlay .w2ui-select-field table td:nth-child(1) input {
  margin: 3px 2px 2px 2px;
}
.w2ui-overlay .w2ui-select-field table td:nth-child(2) {
  padding: 3px 15px 3px 3px;
}
.w2ui-overlay .w2ui-col-on-off {
  padding: 4px 0px;
}
.w2ui-overlay .w2ui-col-on-off table {
  border-spacing: 0px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-col-on-off table tr:hover {
  background-color: #EECCCC;
}
.w2ui-overlay .w2ui-col-on-off table td input[type=checkbox] {
  margin: 3px 2px 2px 2px;
}
.w2ui-overlay .w2ui-col-on-off table td label {
  display: block;
  padding: 3px 0px;
  padding-right: 10px;
}
.w2ui-overlay .w2ui-col-on-off table td:first-child {
  padding: 4px 0px 4px 6px;
}
.w2ui-overlay .w2ui-col-on-off table td:last-child {
  padding: 4px 6px 4px 0px;
}
.w2ui-overlay .w2ui-grid-searches {
  text-align: left;
  padding: 0px;
  border-top: 0px;
  background-color: #f3f1d9;
}
.w2ui-overlay .w2ui-grid-searches table {
  padding: 4px;
  padding-top: 8px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-grid-searches table td {
  padding: 4px;
}
.w2ui-overlay .w2ui-grid-searches table td.close-btn {
  width: 20px;
  padding-right: 20px;
}
.w2ui-overlay .w2ui-grid-searches table td.close-btn input {
  width: 25px !important;
}
.w2ui-overlay .w2ui-grid-searches table td.caption {
  text-align: right;
  padding-right: 5px;
  border-right: 1px solid #c0c0c0;
}
.w2ui-overlay .w2ui-grid-searches table td.operator {
  text-align: left;
  padding: 0px 10px;
  padding-right: 5px;
  border-right: 1px solid #c0c0c0;
}
.w2ui-overlay .w2ui-grid-searches table td.value {
  padding-right: 5px;
  padding-left: 5px;
  background-color: #f7f6f0;
}
.w2ui-overlay .w2ui-grid-searches table td.value input[type=text] {
  border-radius: 3px;
  padding: 3px;
  margin-right: 3px;
  height: 23px;
}
.w2ui-overlay .w2ui-grid-searches table td.value select {
  padding: 3px;
  margin-right: 3px;
  height: 23px;
}
.w2ui-overlay .w2ui-grid-searches table td.actions {
  border-right: 0px;
  padding: 20px 0px 10px 0px;
  text-align: center;
}
.w2ui-overlay .w2ui-grid-searches table td input[type=button] {
  width: 70px;
  margin: 0px 2px;
}
/*************************************************
* ---- Popup ----
*/
.w2ui-popup {
  position: fixed;
  z-index: 1600;
  overflow: hidden;
  font-family: Verdana, Arial;
  border-radius: 6px;
  padding: 0px;
  margin: 0px;
  border: 1px solid #777777;
  background-color: #eeeeee;
  box-shadow: 0px 0px 25px #555555;
}
.w2ui-popup,
.w2ui-popup * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-popup .w2ui-msg-title {
  padding: 6px;
  border-radius: 6px 6px 0px 0px;
  background-image: -webkit-linear-gradient(#ececec, #dfdfdf);
  background-image: -moz-linear-gradient(#ececec, #dfdfdf);
  background-image: -ms-linear-gradient(#ececec, #dfdfdf);
  background-image: -o-linear-gradient(#ececec, #dfdfdf);
  background-image: linear-gradient(#ececec, #dfdfdf);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffececec', endColorstr='#ffdfdfdf', GradientType=0);
  border-bottom: 2px solid #bfbfbf;
  position: absolute;
  overflow: hidden;
  height: 32px;
  left: 0px;
  right: 0px;
  top: 0px;
  text-overflow: ellipsis;
  text-align: center;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
  cursor: move;
  font-size: 15px;
  color: #555555;
  z-index: 300;
}
.w2ui-popup .w2ui-msg-button {
  float: right;
  width: 18px;
  height: 18px;
  cursor: pointer;
  overflow: hidden;
  padding: 0px;
  margin: 0px 3px 0px 0px;
  background: url(data:image/png;base64, ) no-repeat center left;
  background-position: 0px 0px;
  color: transparent !important;
  border-radius: 3px;
  border: 1px solid transparent;
}
.w2ui-popup .w2ui-msg-close {
  margin-top: 0px;
  background-position: -32px 0px;
}
.w2ui-popup .w2ui-msg-close:hover {
  background-color: #cccccc;
  border: 1px solid #aaaaaa;
}
.w2ui-popup .w2ui-msg-max {
  background-position: -16px 0px;
}
.w2ui-popup .w2ui-msg-max:hover {
  background-color: #cccccc;
  border: 1px solid #aaaaaa;
}
.w2ui-popup .w2ui-box1,
.w2ui-popup .w2ui-box2 {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 32px;
  bottom: 35px;
  z-index: 100;
}
.w2ui-popup .w2ui-msg-body {
  font-size: 13px;
  line-height: 130%;
  padding: 0px 7px 7px 7px;
  color: #000000;
  background-color: #eeeeee;
  position: absolute;
  overflow: auto;
  width: 100%;
  height: 100%;
}
.w2ui-popup .w2ui-popup-message {
  position: absolute;
  z-index: 250;
  background-color: #f9f9f9;
  border: 1px solid #999999;
  box-shadow: 0px 0px 15px #aaaaaa;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  border-top: 0px;
  border-radius: 0px 0px 6px 6px;
  overflow: auto;
}
.w2ui-popup .w2ui-msg-buttons {
  padding: 8px;
  border-radius: 0px 0px 6px 6px;
  border-top: 1px solid #d5d8d8;
  background-color: #f1f1f1;
  text-align: center;
  position: absolute;
  overflow: hidden;
  height: 42px;
  left: 0px;
  right: 0px;
  bottom: 0px;
  z-index: 200;
}
.w2ui-popup .w2ui-popup-button {
  width: 70px;
  margin: 0px 5px;
}
.w2ui-popup .w2ui-msg-no-title {
  border-top-left-radius: 6px;
  border-top-right-radius: 6px;
  top: 0px !important;
}
.w2ui-popup .w2ui-msg-no-buttons {
  border-bottom-left-radius: 6px;
  border-bottom-right-radius: 6px;
  bottom: 0px !important;
}
/*************************************************
* ---- Sidebar ----
*/
.w2ui-sidebar {
  cursor: default;
  overflow: hidden !important;
  background-color: #ffffff !important;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-sidebar * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-sidebar > div {
  position: relative;
  overflow: hidden;
}
.w2ui-sidebar .w2ui-sidebar-top {
  position: absolute;
  z-index: 2;
  top: 0px;
  left: 0px;
  right: 0px;
}
.w2ui-sidebar .w2ui-sidebar-bottom {
  position: absolute;
  z-index: 2;
  bottom: 0px;
  left: 0px;
  right: 0px;
}
.w2ui-sidebar .w2ui-sidebar-div {
  position: absolute;
  z-index: 1;
  overflow: auto;
  top: 0px;
  bottom: 0px;
  left: 0px;
  right: 0px;
  padding: 2px 0px;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
  width: 100%;
}
.w2ui-sidebar .w2ui-sidebar-div table {
  width: 100%;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node {
  background-image: -webkit-linear-gradient(#ffffff, #ffffff);
  background-image: -moz-linear-gradient(#ffffff, #ffffff);
  background-image: -ms-linear-gradient(#ffffff, #ffffff);
  background-image: -o-linear-gradient(#ffffff, #ffffff);
  background-image: linear-gradient(#ffffff, #ffffff);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffffffff', endColorstr='#ffffffff', GradientType=0);
  margin: 0px;
  padding: 1px 0px;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node table {
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node td.w2ui-node-dots {
  color: #000000;
  text-shadow: 0px 0px 0px #ffffff;
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-caption:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node td.w2ui-node-dots:hover {
  color: inherit;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node:hover {
  background-image: -moz-linear-gradient(#EECCCC, #EECCCC);
  background-image: -ms-linear-gradient(#EECCCC, #EECCCC);
  background-image: -o-linear-gradient(#EECCCC, #EECCCC);
  background-image: linear-gradient(#EECCCC, #EECCCC);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffEECCCC', endColorstr='#ffEECCCC', GradientType=0);
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image {
  width: 22px;
  text-align: center;
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span {
  color: #516173 !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover {
  background-image: -webkit-linear-gradient(#ff3333, #BB3333);
  background-image: -moz-linear-gradient(#ff3333, #BB3333);
  background-image: -ms-linear-gradient(#ff3333, #BB3333);
  background-image: -o-linear-gradient(#ff3333, #BB3333);
  background-image: linear-gradient(#ff3333, #BB3333);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffff3333', endColorstr='#ffBB3333', GradientType=0);
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected td.w2ui-node-dots,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover td.w2ui-node-dots {
  color: #ffffff !important;
  text-shadow: 1px 1px 2px #666666 !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover {
  background-image: -webkit-linear-gradient(#ffffff, #ffffff);
  background-image: -moz-linear-gradient(#ffffff, #ffffff);
  background-image: -ms-linear-gradient(#ffffff, #ffffff);
  background-image: -o-linear-gradient(#ffffff, #ffffff);
  background-image: linear-gradient(#ffffff, #ffffff);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffffffff', endColorstr='#ffffffff', GradientType=0);
  border-top: 1px solid transparent;
  border-bottom: 1px solid transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled td.w2ui-node-dots,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover td.w2ui-node-dots {
  opacity: 0.4;
  filter: alpha(opacity=40);
  color: #000000 !important;
  text-shadow: 0px 0px 0px #ffffff !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-caption {
  white-space: nowrap;
  padding: 5px 0px 5px 3px;
  margin: 1px 0px 1px 22px;
  position: relative;
  z-index: 1;
  font-size: 12px;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group {
  white-space: nowrap;
  overflow: hidden;
  padding: 10px 0px 10px 10px;
  margin: 0px;
  cursor: default;
  color: #868b92;
  background-color: transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group :nth-child(1) {
  /* show / hide link */
  margin-right: 10px;
  float: right;
  color: transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group :nth-child(2) {
  /* title text */
  font-weight: normal;
  text-transform: uppercase;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-sub {
  overflow: hidden;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-dots {
  width: 18px;
  padding: 0px 0px 1px 7px;
  text-align: center;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-dots .w2ui-expand {
  width: 16px;
  margin-top: -3px;
  pointer-events: auto;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data {
  padding: 1px 1px 3px 1px;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image {
  padding: 3px 0px 0px 0px;
  float: left;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image > span {
  font-size: 16px;
  color: #000000;
  text-shadow: 0px 0px 0px #ffffff;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image.w2ui-icon {
  margin-top: 3px;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-count {
  float: right;
  border: 1px solid #9da4af;
  border-radius: 20px;
  width: auto;
  height: 18px;
  padding: 2px 7px;
  margin: 3px 4px -2px 0;
  background-color: #e7f0fc;
  color: #667274;
  box-shadow: 0 0 2px #ffffff;
  text-shadow: 1px 1px 1px #e6e6e6;
  position: relative;
  z-index: 2;
}
/*************************************************
* ---- Tabs ----
*/
.w2ui-tabs {
  cursor: default;
  overflow: hidden !important;
  background-color: #fafafa;
  padding: 3px 0px;
  padding-bottom: 0px !important;
}
.w2ui-tabs table {
  border-bottom: 1px solid silver;
  padding: 0px 7px;
}
.w2ui-tabs .w2ui-tab {
  padding: 6px 20px;
  text-align: center;
  color: #000000;
  background-color: transparent;
  border: 1px solid #c0c0c0;
  border-bottom: 1px solid silver;
  white-space: nowrap;
  margin: 1px 1px -1px 0px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  cursor: default;
}
.w2ui-tabs .w2ui-tab.active {
  color: #000000;
  background-color: #ffffff;
  border: 1px solid #c0c0c0;
  border-bottom: 1px solid transparent;
}
.w2ui-tabs .w2ui-tab.closable {
  padding: 6px 28px 6px 20px;
}
.w2ui-tabs .w2ui-tab-close {
  color: #555;
  text-shadow: 1px 1px 1px #bbb;
  float: right;
  margin: 6px 4px 0px 0px;
  padding: 0px 0px 0px 5px;
  width: 16px;
  height: 16px;
  opacity: 0.9;
  border: 0px;
  border-top: 3px solid transparent;
  border-radius: 9px;
}
.w2ui-tabs .w2ui-tab-close:hover {
  background-color: #D77F7F;
  color: white;
}
.w2ui-tabs .w2ui-tab-close:before {
  position: relative;
  top: -2px;
  left: 0px;
  opacity: 0.6;
  color: inherit;
  text-shadow: inherit;
  content: 'x';
}
/*************************************************
* ---- Toolbar ----
*/
.w2ui-toolbar {
  margin: 0px;
  padding: 2px;
  outline: 0px;
  background-color: #efefef;
  overflow: hidden !important;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-toolbar .disabled {
  opacity: 0.3;
  filter: alpha(opacity=30);
}
.w2ui-toolbar table {
  table-layout: auto !important;
}
.w2ui-toolbar table td {
  border: 0px !important;
}
.w2ui-toolbar table.w2ui-button {
  margin: 0px 1px;
  border-radius: 4px;
  height: 24px;
  border: 1px solid transparent;
  background-color: transparent;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-image {
  width: 16px;
  height: 16px;
  padding: 0px;
  margin: 3px 3px 3px 3px !important;
  border: 0px !important;
  text-align: center;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-image > span {
  font-size: 16px;
  color: #516173;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-caption {
  color: #000000;
  padding: 0px 4px 0px 2px;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-down {
  width: 12px;
  height: 24px;
  display: block;
  background: url(data:image/png;base64,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) no-repeat center right;
}
.w2ui-toolbar table.w2ui-button.over {
  border: 1px solid #cccccc;
  background-color: #eeeeee;
}
.w2ui-toolbar table.w2ui-button.over .w2ui-tb-caption {
  color: #000000;
}
.w2ui-toolbar table.w2ui-button.down {
  border: 1px solid #aaaaaa;
  background-color: #dddddd;
}
.w2ui-toolbar table.w2ui-button.down .w2ui-tb-caption {
  color: #666666;
}
.w2ui-toolbar table.w2ui-button.checked {
  border: 1px solid #aaaaaa;
  background-color: #ffffff;
}
.w2ui-toolbar table.w2ui-button.checked .w2ui-tb-caption {
  color: #000000;
}
.w2ui-toolbar table.w2ui-button table {
  height: 17px;
  border-radius: 4px;
  cursor: default;
}
.w2ui-toolbar .w2ui-break {
  background-image: -webkit-linear-gradient(top, rgba(153, 153, 153, 0.09999999999999998) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.09999999999999998) 100%);
  background-image: -moz-linear-gradient(top, rgba(153, 153, 153, 0.09999999999999998) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.09999999999999998) 100%);
  background-image: -ms-linear-gradient(top, rgba(153, 153, 153, 0.09999999999999998) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.09999999999999998) 100%);
  background-image: -o-linear-gradient(top, rgba(153, 153, 153, 0.09999999999999998) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.09999999999999998) 100%);
  background-image: linear-gradient(top, rgba(153, 153, 153, 0.09999999999999998) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.09999999999999998) 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff999999', endColorstr='#ff999999', GradientType=0);
  width: 1px !important;
  height: 22px;
  padding: 0px;
  margin: 0px 6px;
}
.w2ui-overlay table.w2ui-drop-menu {
  width: 100%;
  color: #000000;
  background-color: #ffffff;
  padding: 5px 0px;
  cursor: default;
}
.w2ui-overlay table.w2ui-drop-menu td:first-child {
  padding: 4px 0px 4px 6px;
}
.w2ui-overlay table.w2ui-drop-menu td:last-child {
  padding: 10px 15px 10px 5px;
}
.w2ui-overlay table.w2ui-drop-menu tr.w2ui-selected {
  background-color: #EECCCC;
}
.w2ui-overlay table.w2ui-drop-menu tr.w2ui-selected td {
  color: inherit;
}
.w2ui-overlay table.w2ui-drop-menu .w2ui-tb-image > span {
  font-size: 16px;
  color: #516173;
  display: inline-block;
  padding-top: 4px;
}
